// Definitions used with language bindings

#ifndef IDLC_LANG_H
#define IDLC_LANG_H

#include <idlc.h>

// A subclass of this interface is passed to Symbol::output_lang(),
// which calls the appropriate type-specific method here.  If the
// sym is a Datum, Method, or Param, no callback is made; the language
// binding is expected to handle it when iterating over the containing
// type's element list.

class LangCallback {
public:
	virtual ~LangCallback()
	{
	}

	virtual void output(UserNameSpace *sym, int arg1, void *arg2) = 0;
	virtual void output(Struct *sym, int arg1, void *arg2) = 0;
	virtual void output(Interface *sym, int arg1, void *arg2) = 0;
	virtual void output(BitField *sym, int arg1, void *arg2) = 0;
	virtual void output(Enum *sym, int arg1, void *arg2) = 0;
	virtual void output(BasicType *sym, int arg1, void *arg2) = 0;
	virtual void output(Alias *sym, int arg1, void *arg2) = 0;
	virtual void output(TypeDef *sym, int arg1, void *arg2) = 0;
	virtual void output(Datum *sym, int arg1, void *arg2) = 0; // const only
};

class Language {
public:
	const char *name;
	Language *next;

	virtual ~Language()
	{
	}
	
	virtual void output_root(UserNameSpace *ns, const char *dir) = 0;
	virtual void output_server(UserNameSpace *ns, const char *dir) = 0;
};

extern Language *first_lang;

#endif
